; poznamka - symbol (*) znaci, ze velicina je pouze adresou promenne

KEY_INIT	MACRO	IREGISTR, EIBIT, IBIT, PORTX, KB, K_OUT, KROK, K_STATUS, K_CAS1, K_CAS2, S_KLAV
; inicializace klavesnice
; IREGISTR - adresa registru, ktery obsahuje priznak nebo povoleni preruseni
;            pro zvoleny casovac
; EIBIT - bit v IREGISTRu, ktery povoluje preruceni od casovace
; IBIT - bit v IREGISTRu, ktery indikuje preruseni
; PORTX - adresa 8pinoveho portu, ke kteremu je pripojena klavesnice
; KB* - 2bytova pomocna promenna
; K_OUT* - 2bytova vystupni promenna
; KROK* - pomocna promenna
; K_STATUS* - promenna, ktera obsahuje stav obsluhy
;              (bit 0 rika, zda doslo ke zmene stavu klavesnice)
; S_KLAV* - minuly stav klavesnice
; K_CAS1* - promenna specialniho casovace (viz. odstranovani zakmitu)
; K_CAS2* - promenna specialniho casovace (viz. repetice klaves)

	banksel	D'128'
	BSF	IREGISTR,EIBIT	; povoleni preruseni od casovace
	MOVLW	B'11101111'	; nastaveni TRIS registru pro PORTX (take banka 1)
	MOVWF	PORTX
	banksel	0
	MOVLW	B'00000000'	; nastaveni PORT registru (na nulovy potencial)
	MOVWF	PORTX
	CLRF	KB+1		; vynulovani pomocnych promennych
	CLRF	KB
	CLRF	K_OUT+1
	CLRF	K_OUT
	CLRF	KROK
	CLRF	K_STATUS
	CLRF	K_CAS1
	MOVLW	D'10'
	MOVWF	K_CAS2
	CLRF	S_KLAV
	BCF	IREGISTR, IBIT	; vynulovani priznaku od casovace

	ENDM


KEY_IPROG	MACRO	IREGISTR, IBIT, PORTX, KB, KB2, KBCITAC, K_OUT, KROK, XTEMP, K_STATUS, S_KLAV, K_CAS1, K_CAS2, T_STABIL, T_REPET1, T_REPET2, T_R_PRED
; obsluha klavesnice (pro system preruseni)
; IREGISTR - adresa registru, ktery obsahuje priznak preruseni
;            pro zvoleny casovac
; IBIT - bit v IREGISTRu, ktery indikuje preruseni
; PORTX - adresa 8pinoveho portu, ke kteremu je pripojena klavesnice
; KB* - 2bytova pomocna promenna
; KB2* - 4bytova pomocna promenna
; KBCITAC - pomocna promenna
; K_OUT* - 2bytova vystupni promenna
; KROK* - pomocna promenna
; XTEMP* - pomocna promenna
; K_STATUS* - promenna, ktera obsahuje stav obsluhy
;	     (bit 0 indikuje, zda doslo ke zmene stavu klavesnice,
;	      bit 1 indikuje, zda doslo ke zmene z 1 na 0,
;	      bit 2 indikuje, zda doslo k pretizeni klavesnice (dekce "L"))
; S_KLAV* - minuly stav klavesnice
; K_CAS1* - promenna specialniho casovace (viz. odstranovani zakmitu)
; K_CAS2* - promenna specialniho casovace (2 byty) (viz. repetice klaves)
; T_STABIL - nutny pocet cyklu casovace K_CAS1, pro dozneni zakmitu
; T_REPET1 - delka delsi prodlevy repetice
; T_REPET2 - delka kratsi prodlevy repetice
; T_R_PRED - nastaveni predelicky k K_CAS2

	LOCAL	KONEC, K_START, K_ZMENA, K_PROCES
	LOCAL	K_NEXT1, K_NEXT2, K_NEXT3, K_NEXT4, K_NEXT5
	LOCAL	L_TEST, K_CHYBA, K_UKONCI

	banksel	IREGISTR		; toto je stejne jako u minule verze
	BTFSS	IREGISTR,IBIT
	GOTO	KONEC
	BCF	IREGISTR,IBIT
	MOVF	K_CAS1,F
	BTFSC	STATUS,Z
	GOTO	K_START
	DECF	K_CAS1,F
	GOTO	KONEC
K_START	MOVF	PORTX,W
	ANDLW	B'00001111'
	BTFSC	KROK,0
	SWAPF	PORTX,W
	BTFSC	KROK,0
	ANDLW	B'11110000'
	BTFSS	KROK,1
	IORWF	KB,F
	BTFSC	KROK,1
	IORWF	KB+1,F

	INCF	KROK,F
	MOVLW	B'11101111'
	MOVWF	XTEMP
	BSF	STATUS,C
	BTFSC	KROK,0
	RLF	XTEMP,F
	BTFSC	KROK,1
	RLF	XTEMP,F
	BTFSC	KROK,1
	RLF	XTEMP,F
	MOVF	XTEMP,W
	IORLW	B'00001111'
	BSF	STATUS,RP0
	MOVWF	PORTX
	BCF	STATUS,RP0
	
	MOVF	KROK,W
	ANDLW	B'00000011'
	BTFSS	STATUS,Z
	GOTO	KONEC

	DECF	K_CAS2+1,W	; predelicka ke K_CAS2
	ANDLW	T_R_PRED
	MOVWF	K_CAS2+1
	BTFSS	STATUS,Z	; pokud predelicka pretekla svuj rozsah...
	GOTO	$ + 4
	MOVF	K_CAS2,F	; ...a K_CAS2 je vetsi nez nula, potom jej snizime o jednicku
	BTFSS	STATUS,Z
	DECF	K_CAS2,F
	
	MOVF	KB,W	; pokud neni stisknuta zadna klavesa,
	ANDWF	KB+1,W	; potom nastavime K_CAS2 na T_REPET1
	XORLW	B'11111111'
	MOVLW	T_REPET1	; to znamena, ze do repetice zbyva minimalne "dlouha doba"
	BTFSC	STATUS,Z
	MOVWF	K_CAS2

	MOVF	KB,W	; zde se testuje, zda doslo ke zmene
	XORWF	S_KLAV,W	; (stejne jako v minule verzi)
	BTFSS	STATUS,Z
	GOTO	K_ZMENA
	MOVF	KB+1,W
	XORWF	S_KLAV+1,W
	BTFSS	STATUS,Z
	GOTO	K_ZMENA
	MOVF	K_CAS2,F
	BTFSS	STATUS,Z	; pokud ke zmene nedoslo, ale K_CAS2 je nulovy,
	GOTO	K_UKONCI
	MOVLW	T_REPET2	; ...potom nastavime do K_CAS1 kratkou prodlevu
	MOVWF	K_CAS2
	MOVLW	B'11111111'	; nasimulujeme, ze v minulem stavu (S_KLAV) jsou...
	MOVWF	S_KLAV	; ...same jednicky (zadna klavesa nebyla stisknuta)
	MOVWF	S_KLAV+1
	GOTO	K_PROCES	; tvarime se, jako by byla detekovana zmena
K_ZMENA	MOVLW	T_REPET1	; pokud je ale zmena detekovana doopravdy,
	MOVWF	K_CAS2	; potom musime do K_CAS2 ulozit dlouhou prodlevu,...
	BSF	K_STATUS,0	; ...indikovat skutecnou zmenu
	MOVLW	T_STABIL	; a nastavit K_CAS1 pro odstraneni zakmitu
	MOVWF	K_CAS1

K_PROCES	MOVF	KB,W	; odtud je cela obsluha stejna jako v minule verzi
	XORLW	B'11111111'
	ANDLW	B'00001111'
	MOVWF	KB2	
	SWAPF	KB,W
	XORLW	B'11111111'
	ANDLW	B'00001111'
	MOVWF	KB2+1
	MOVF	KB+1,W
	XORLW	B'11111111'
	ANDLW	B'00001111'
	MOVWF	KB2+2	
	SWAPF	KB+1,W
	XORLW	B'11111111'
	ANDLW	B'00001111'
	MOVWF	KB2+3

	MOVF	KB2+0,W
	IORWF	KB2+1,W
	IORWF	KB2+2,W
	ANDWF	KB2+3,W
	BTFSC	STATUS,Z
	GOTO	K_NEXT1
	MOVF	KB2+3,W
	CALL	L_TEST
	BTFSS	STATUS,Z
	GOTO	K_CHYBA
K_NEXT1	MOVF	KB2+1,W
	IORWF	KB2+2,W
	IORWF	KB2+3,W
	ANDWF	KB2+0,W
	BTFSC	STATUS,Z
	GOTO	K_NEXT2
	MOVF	KB2+0,W
	CALL	L_TEST
	BTFSS	STATUS,Z
	GOTO	K_CHYBA
K_NEXT2	MOVF	KB2+2,W
	IORWF	KB2+3,W
	IORWF	KB2+0,W
	ANDWF	KB2+1,W
	BTFSC	STATUS,Z
	GOTO	K_NEXT3
	MOVF	KB2+1,W
	CALL	L_TEST
	BTFSS	STATUS,Z
	GOTO	K_CHYBA
K_NEXT3	MOVF	KB2+3,W
	IORWF	KB2+0,W
	IORWF	KB2+1,W
	ANDWF	KB2+2,W
	BTFSC	STATUS,Z
	GOTO	K_NEXT4
	MOVF	KB2+2,W
	CALL	L_TEST
	BTFSS	STATUS,Z
	GOTO	K_CHYBA

K_NEXT4	MOVF	KB,W
	XORLW	B'11111111'
	MOVWF	KB2
	MOVF	KB+1,W
	XORLW	B'11111111'
	MOVWF	KB2+1
	MOVF	KB,W
	XORWF	S_KLAV,W
	ANDWF	KB2,W
	BTFSS	STATUS,Z
	BSF	K_STATUS,1
	IORWF	K_OUT,F
	MOVF	KB+1,W
	XORWF	S_KLAV+1,W
	ANDWF	KB2+1,W
	BTFSS	STATUS,Z
	BSF	K_STATUS,1
	IORWF	K_OUT+1,F

K_NEXT5	MOVF	KB,W
	MOVWF	S_KLAV
	MOVF	KB+1,W
	MOVWF	S_KLAV+1
	GOTO	K_UKONCI

L_TEST	CLRF	KBCITAC
	MOVWF	XTEMP
	RRF	XTEMP,F
	BTFSC	STATUS,C
	INCF	KBCITAC,F
	RRF	XTEMP,F
	BTFSC	STATUS,C
	INCF	KBCITAC,F
	RRF	XTEMP,F
	BTFSC	STATUS,C
	INCF	KBCITAC,F
	RRF	XTEMP,F
	BTFSC	STATUS,C
	INCF	KBCITAC,F
	DECF	KBCITAC,F
	RETURN
K_CHYBA	BSF	K_STATUS,2
	GOTO	K_NEXT5

K_UKONCI	CLRF	KB
	CLRF	KB+1
	
KONEC
	ENDM
